/**
 * Copyright 2003 DFKI GmbH.
 * All Rights Reserved.  Use is subject to license terms.
 *
 * This file is part of MARY TTS.
 *
 * MARY TTS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package marytts.language.sc;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;
import java.util.regex.*;

import marytts.datatypes.MaryData;
import marytts.datatypes.MaryDataType;
import marytts.datatypes.MaryXML;
import marytts.util.dom.MaryDomUtils;
import marytts.util.dom.NameNodeFilter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;

public class JTokeniser extends marytts.modules.JTokeniser {

	public JTokeniser() {
		super(MaryDataType.RAWMARYXML, MaryDataType.TOKENS, new Locale("sc"));
	}

	public MaryData process(MaryData d) throws Exception {
		MaryData result = super.process(d);
		tokenizerFixes(result);
		return result;
	}

	protected void tokenizerFixes(MaryData d) {
		Document doc = d.getDocument();
		NodeIterator ni = ((DocumentTraversal) doc).createNodeIterator(doc, NodeFilter.SHOW_ELEMENT, new NameNodeFilter(
				MaryXML.TOKEN), false);
		Element t = (Element) ni.nextNode();
		while (t != null) {
			Element t1 = (Element) ni.nextNode();
			String s = MaryDomUtils.tokenText(t);
			if("€".equals(s)) {
				MaryDomUtils.setTokenText(t, "èuro");
			} else if ("%".equals(s)) {
				MaryDomUtils.setTokenText(t, "prochèntu");
			}
			else if ("£".equals(s)) {
				MaryDomUtils.setTokenText(t, "lìras");
			}
			else if ("&".equals(s)) {
				MaryDomUtils.setTokenText(t, "è");
			}
			else if ("=".equals(s)) {
				MaryDomUtils.setTokenText(t, "uguàle");
			}
			else if ("+".equals(s)) {
				MaryDomUtils.setTokenText(t, "prùs");
			}
			else if ("*".equals(s)) {
				MaryDomUtils.setTokenText(t, "asterìscu");
			}
			else if ("@".equals(s)) {
				MaryDomUtils.setTokenText(t, "èt");
			}
			else if ("#".equals(s)) {
				MaryDomUtils.setTokenText(t, "canzellèddu");
			}
			else if ("°".equals(s)) {
				MaryDomUtils.setTokenText(t, "ò");
			}
			else if (">".equals(s)) {
				MaryDomUtils.setTokenText(t, "magiòre");
			}
			else if ("<".equals(s)) {
				MaryDomUtils.setTokenText(t, "minòre");
			}
			else if(s.matches(".*[aeiouAEIOU].*[aeiouAEIOU].*") && !s.matches(".*[àèìòùÀÈÌÒÙúíáéóÁÍÉÓÚ].*")) {
				Pattern pattern = Pattern.compile(".*([aeiouAEIOU])[^aeiouAEIOU]*[aeiouAEIOU][^aeiouAEIOU]*");
				Matcher m = pattern.matcher(s);
				if (m.find()) {
					String replace = "";
					switch(m.group(1)) {
						case "a":
							replace = "à";
							break;
						case "e":
							replace = "è";
							break;
						case "i":
							replace = "ì";
							break;
						case "o":
							replace = "ò";
							break;
						case "u":
							replace = "ù";
							break;
						case "A":
							replace = "À";
							break;
						case "E":
							replace = "È";
							break;
						case "I":
							replace = "Ì";
							break;
						case "O":
							replace = "Ò";
							break;
						case "U":
							replace = "Ù";
							break;
					}
					int retIndex = m.end(1);
					MaryDomUtils.setTokenText(t, s.substring(0, m.start(1)) + replace + s.substring(m.end(1), s.length()));
				}
			}
			if (t1 == null && !s.endsWith(".") && !s.endsWith("?") && !s.endsWith("!")) {
				Element sentence = (Element) MaryDomUtils.getAncestor(t, MaryXML.SENTENCE);
				Element newT = MaryXML.appendChildElement(sentence, MaryXML.TOKEN);
				MaryDomUtils.setTokenText(newT, ".");
			}
			t = t1;
		}
	}
}
