/**
 * Copyright 2002 DFKI GmbH.
 * All Rights Reserved.  Use is subject to license terms.
 *
 * This file is part of MARY TTS.
 *
 * MARY TTS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package marytts.language.sc;

import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import marytts.datatypes.MaryData;
import marytts.datatypes.MaryXML;
import marytts.modules.PronunciationModel;
import marytts.util.dom.MaryDomUtils;
import marytts.util.dom.NameNodeFilter;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.TreeWalker;

public class Postlex extends PronunciationModel {

	public Postlex() {
		super(new Locale("sc"));
	}

	public MaryData process(MaryData d) throws Exception {
		Document doc = d.getDocument();
		phonologicalRules(doc);
		return super.process(d);
	}

	private void phonologicalRules(Document doc) {
		NodeList allTokens = doc.getElementsByTagName(MaryXML.TOKEN);
		/**
		 * Metafonesi 
		 */
		for (int i = 0; i < allTokens.getLength(); i++) {
			Element t = (Element) allTokens.item(i);
			if (t.hasAttribute("ph") && (!t.hasAttribute("g2p_method") || !t.getAttribute("g2p_method").equals("lexicon"))) { // otherwise there's no point
				String s = t.getAttribute("ph");
				String[] tokens = s.split(" ");
				String new_s = "";
				System.err.println("Meta " + s);
				boolean close = false;
				for(int j = tokens.length -1; j >= 0; j--) {
					System.err.println(tokens[j]);
					if(tokens[j].matches("(a|aa|ah)")) {
						System.out.println("open: " + tokens[j]);
						close = false;
					}
					else if(tokens[j].matches("(ii|i|ih|uu|u|uh|j|x|S|G|C|N)")) {
						System.out.println("close: " + tokens[j]);
						close = true;
					}
					else if(close && tokens[j].matches("(EE|E|Eh|OO|O|Oh)")) {
						System.out.println("closing: " + tokens[j]);
						if(tokens[j].equals("EE")) {
							tokens[j] = "ee";
						}
						else if(tokens[j].equals("E") || tokens[j].equals("Eh")) {
							tokens[j] = "e";
						}
						else if(tokens[j].equals("OO")) {
							tokens[j] = "oo";
						}
						else if(tokens[j].equals("O") || tokens[j].equals("Oh")) {
							tokens[j] = "o";
						}
					}
					else if (!close && tokens[j].matches("(ee|e|oo|o)")) {
						System.out.println("opening: " + tokens[j]);
						if(tokens[j].equals("ee")) {
							tokens[j] = "EE";
						}
						else if(tokens[j].equals("e")) {
							tokens[j] = "E";
						}
						else if(tokens[j].equals("oo")) {
							tokens[j] = "OO";
						}
						else if(tokens[j].equals("o")) {
							tokens[j] = "O";
						}
					}
					if(new_s.equals(""))
						new_s = tokens[j];
					else
						new_s = tokens[j] + " " + new_s;
				}
				t.setAttribute("ph", new_s);
				System.err.println(new_s);
			}
		}
		/**
		 * Fonetica sintàtica
		 * */
		for (int i = 0; i < allTokens.getLength(); i++) {
			Element t = (Element) allTokens.item(i);
			Element next = (Element) t.getNextSibling();
			String text = MaryDomUtils.tokenText(t);
			if(text.equals(",")) {
				t.setAttribute("pos", "$,");
			}
			else if(text.equals(".")) {
				t.setAttribute("pos", "$.");
			}
			else if(text.equals("(")) {
				t.setAttribute("pos", "$(");
			}
			else if (t.hasAttribute("ph")) { // otherwise there's no point
				String s = t.getAttribute("ph");
				/* L e R in cumintzu de paràula*/
				if(s.matches("(' )?(l|R|r)(.*)")) {
					if(s.charAt(0) == '\'') {
						t.setAttribute("ph", "' " + s.charAt(2) + " " + s.substring(2, s.length()));
					}
					else {
						t.setAttribute("ph", s.charAt(0) + " " + s);
					}
				}
				else if(s.matches("(' )?ll(.*)")) {
					if(s.charAt(0) == '\'') {
						t.setAttribute("ph", "' " + s.charAt(2) + s.charAt(3) + " " + s.substring(3, s.length()));
					}
					else {
						t.setAttribute("ph", s.charAt(0) + s.charAt(1) + " " + s);
					}
				}
				if(next.hasAttribute("ph")) {
					boolean changed = false;
					String s_next = next.getAttribute("ph");
					String text_next = MaryDomUtils.tokenText(next);
					/*cogeminazione e degeminazione*/
					if(text.matches("(a|che|do|e|fa|o|re|se|tra|A|Che|Do|E|Fa|O|Re|Se|Tra|à|chè|dò|è|fà|ò|rè|sè|trà|À|Chè|Dò|È|Fà|Ò|Rè|Sè|Trà)") && !text_next.matches("(lu|la|los|las|lis|li|Lu|La|Los|Las|Lis|Li|lù|là|lòs|làs|lìs|lì|Lù|Là|Lòs|Làs|Lìs|Lì)")) {
						if(s_next.matches("' (mm|n1|n2|n3|n4|nn|bb|dd|DD|gh|ss|ts|dz|ll|bh|dh)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + s_next.charAt(2) + s_next.charAt(3) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(mm|n1|n2|n3|n4|nn|bb|dd|DD|gh|ss|ts|dz|ll|bh|dh)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + s_next.charAt(0) + s_next.charAt(1) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("' (m|n|N|p|b|t|d|D|k|g|f|v|s|z|S|x|w|C|G|r|R|l|j)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + s_next.charAt(2) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(m|n|N|p|b|t|d|D|k|g|f|v|s|z|S|x|w|C|G|r|R|l|j)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + s_next.charAt(0) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					/*D eufonica*/
					if(text.matches("(cun|Cun|in|In|Ìn|ìn|Cùn|cùn)") && text_next.matches("(unu|una|Unu|Una|ùnu|ùna|Ùnu|Ùna|unos|unas|Unos|Unas|ùnos|ùnas|Ùnos|Ùnas|un'.*|Un'.*|ùn'.*|Ùn'.*)") && !changed) {
						MaryDomUtils.setTokenText(t, text + " " + text_next);
						if(s_next.startsWith("'")) {
							t.setAttribute("ph", s.substring(2, s.length()) + " - ' d " + s_next.substring(2, s_next.length()));
						}
						else {
							t.setAttribute("ph", s.substring(2, s.length()) + " - ' d " + s_next);
						}
						next.getParentNode().removeChild(next);
						changed = true;	
					}
					/*-ere*/
					if(text.matches("(.)*ere") && !changed) {
						t.setAttribute("ph",  s.substring(0, s.length()-1));
						changed = true;	
					}
					/* -st */
					if(s.matches("(.*)(ii|ih|ee|EE|Oh|oo|uu|uh) s t") && !changed) {
						if(s_next.matches("(' )?x(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "s - t " + s.charAt(s.length()-6) + s.charAt(s.length()-5) + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "s - t " + s.charAt(s.length()-6) + s.charAt(s.length()-5) + " j - " +  s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*)(a|i|e|E|O|o|u) s t") && !changed) {
						if(s_next.matches("(' )?x(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "s - t " + s.charAt(s.length()-5) + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "s - t " + s.charAt(s.length()-5) + " j - " +  s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) s t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- t " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?(m|n|N|b|d|D|g|ts|dz|C|G|f|v|r|l)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "R R - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[stpk](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "s - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?S(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "s - t i S - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					/* -s */
					if(s.matches("(.*) s") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- z " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?(m|n|N|b|d|D|g|ts|dz|C|G|f|v|x|j|r|l)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "R R - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?S(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "z i S - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					/* -R */
					if(s.matches("(.*)(ii|ih|ee|EE|Oh|oo|uu|uh) R") && !changed) {
						if(s_next.matches("(' )?x(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s.charAt(s.length()-4) + s.charAt(s.length()-3) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s.charAt(s.length()-4) + s.charAt(s.length()-3) + " j - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*)(i|e|E|O|o|u|a) R") && !changed) {
						if(s_next.matches("(' )?x(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s.charAt(s.length()-3) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s.charAt(s.length()-3) + " j - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) R") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?S(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- r i S - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					/* -nt */
					if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) n t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?(m|n|x|r|l)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " +  s.charAt(s.length()-6) + s.charAt(s.length()-5) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?N(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- N " +  s.charAt(s.length()-6) + s.charAt(s.length()-5) + " N - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s.charAt(s.length()-6) + s.charAt(s.length()-5) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*)(a|i|e|E|O|o|u) n t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?(m|n|x|r|l)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " +  s.charAt(s.length()-5) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?N(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- N " + s.charAt(s.length()-5) + " N - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s.charAt(s.length()-5) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) n t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[pb](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "m - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?(s|t|d|ts|tz)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "n - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?D(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "n3 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[kg](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "n4 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[CG](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "n2 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[fv](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "n1 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?S(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-3) + "n i S - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					/* -n */
					if(s.matches("(.*) n") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- n " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?N(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "N - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[mpb](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "m - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[Dj](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "n3 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[kg](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "n4 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[CGsX](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "n2 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?[fv](.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "n1 - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					/* -t */
					if(s.matches("(.*)(ii|ih|ee|EE|Oh|oo|uu|uh) t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?x(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s.charAt(s.length()-4) + s.charAt(s.length()-3) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s.charAt(s.length()-4) + s.charAt(s.length()-3) + " j - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*)(i|e|E|O|o|u|a) t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?x(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s.charAt(s.length()-3) + " - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s.charAt(s.length()-3) + " j - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) t") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?m(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "m - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?n(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "n - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?N(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "N - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?p(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "p - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?b(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "b - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?d(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "d - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?D(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "D - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?k(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "k - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?g(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "g - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?ts(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "ts - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?dz(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "dz - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?C(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "C - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?G(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "G - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?f(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "f - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?v(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "v - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?s(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "s - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?S(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "dh i S - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?r(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "R - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?l(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s.substring(0, s.length()-1) + "l - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) (ts)") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ts " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) (S)") && !changed) {
						if(s_next.matches("(' )?(ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - S" + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
					if(s.matches("(.*) (ii|ih|ee|EE|Eh|Oh|oo|uu|uh|i|e|E|O|o|u|a)") && !changed) {
						if(s_next.matches("' t(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ' dh " + s_next.substring(3));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("t(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - dh " + s_next.substring(1));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("' p(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ' bh " + s_next.substring(3));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("p(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - bh " + s_next.substring(1));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("' k(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ' gh " + s_next.substring(3));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("k(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - gh " + s_next.substring(1));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("' f(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ' v " + s_next.substring(3));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("f(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - v " + s_next.substring(1));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("' s(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ' z " + s_next.substring(3));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("s(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - z " + s_next.substring(1));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("' R(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - ' r " + s_next.substring(3));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("R(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " - r " + s_next.substring(1));
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?N(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " N - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?ts(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " ts - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?dz(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " dz - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?C(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " C - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?G(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " G - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?S(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " S - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
						else if(s_next.matches("(' )?j(.*)")) {
							MaryDomUtils.setTokenText(t, text + " " + text_next);
							t.setAttribute("ph", s + " j - " + s_next);
							next.getParentNode().removeChild(next);
							changed = true;
						}
					}
				}
				t = (Element) allTokens.item(i);
				next = (Element) t.getNextSibling();
				text = MaryDomUtils.tokenText(t);
				s = t.getAttribute("ph");
				if(next == null || !next.hasAttribute("ph")) {
					if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) n")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- n " + s.charAt(s.length()-4) + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(a|i|e|E|O|o|u) n")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- n " + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) n t")) {
						t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s.charAt(s.length()-6) + s.charAt(s.length()-5));
					}
					else if(s.matches("(.*)(a|i|e|E|O|o|u) n t")) {
						t.setAttribute("ph", s.substring(0, s.length()-3) + "- n " + s.charAt(s.length()-5));
					}
					else if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) R")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s.charAt(s.length()-4) + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(a|i|e|E|O|o|u) R")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- r " + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) s")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- z " + s.charAt(s.length()-4) + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(a|i|e|E|O|o|u) s")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- z " + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) s t")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- t " + s.charAt(s.length()-6) + s.charAt(s.length()-5));
					}
					else if(s.matches("(.*)(a|i|e|E|O|o|u) s t")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- t " + s.charAt(s.length()-5));
					}
					else if(s.matches("(.*)(ii|ih|ee|EE|Eh|Oh|oo|uu|uh) t")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s.charAt(s.length()-4) + s.charAt(s.length()-3));
					}
					else if(s.matches("(.*)(a|i|e|E|O|o|u) t")) {
						t.setAttribute("ph", s.substring(0, s.length()-1) + "- dh " + s.charAt(s.length()-3));
					}
				}
			}
		} // for all tokens
	}

}
